/**
 * @file LED_Flash_click_ARM.c
 * @brief <h2> LED Flash click Example </h2>
 *
 * @par
 * Example for <a href="http://www.mikroe.com">MikroElektronika's</a>
 * LED Flash click board.
 *
 *
 *
 ******************************************************************************/

/**
 * @page Example
 * @date        08/11/2016
 * @author      Branislav Marton
 * @copyright   GNU Public License
 * @version     1.0.0 - Initial testing and verification
 */

/**
 * @page TEST_CFG Test Configurations
 *
 * ### Test configuration STM : ###
 * @par
 * -<b> MCU           </b> :    STM32F207VG
 * -<b> Dev. Board    </b> :    EasyMx PRO v7 for STM32
 * -<b> Oscillator    </b> :
 * -<b> Ext. Modules  </b> :    LED Flash Click Board
 * -<b> SW            </b> :    MicroC PRO for ARM v4.9.0
 *
 */

  /**
 * @page Dev. board setup
 *
 * 1) Put LED Flash click in mikroBUS 1 slot.
 * 2) Button press level for PORTC/H and PORTD/L set to GND.
 * 3) Set pull-up for PD4 and PC13.
 */

/**************************************************************************
* Includes
**************************************************************************/
#include <stdint.h>
#include "tft_resources.h"

/**************************************************************************
* Module Preprocessor Constants
**************************************************************************/
#define POLLING_PERIOD 10

#define MODE_TEXT_Y 75
#define TORCH_TEXT_X 20
#define FLASH_TEXT_X 100
#define READY_TEXT_X 180

/**************************************************************************
* Module Typedefs
**************************************************************************/
typedef enum state 
{
    STATE_READY,
    STATE_CHARGE,
    STATE_TORCH,
    STATE_FLASH
} State;

/**************************************************************************
* Module Variable Definitions
**************************************************************************/
sbit FLASH at GPIOC_ODRbits.B2;
sbit TORCH at GPIOA_ODRbits.B0;
sbit CHRG at GPIOA_ODRbits.B4;
sbit FLAG at GPIOD_IDRbits.B10;

// TFT module connections
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_CS at GPIOE_ODR.B15;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_WR at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;
// End TFT module connections


uint8_t buttonstate_center = 0xFF;
uint8_t buttonstate_up = 0xFF;

State state;
State next_state;

/******************************************************************************
* Function Prototypes
*******************************************************************************/
void MCU_Init();
uint8_t Button_Pressed(unsigned int *port, unsigned int pin, uint8_t* state);
void Write_Text(unsigned char* text, unsigned x, unsigned y, unsigned int color);

void Write_Mode_Text(
    uint8_t write_torch,
    unsigned int torch_color, 
    uint8_t write_flash,
    unsigned int flash_color,
    uint8_t write_ready,
    unsigned int ready_colort
);

/******************************************************************************
* Public Functions
*******************************************************************************/
void main() 
{
    MCU_Init();

    state = STATE_CHARGE;
    next_state = STATE_CHARGE;

    while(1) 
    {
        switch(state) 
        {
            case STATE_CHARGE:
                Write_Mode_Text(1, CL_GRAY, 1, CL_BLACK, 1, CL_GRAY);
                FLASH = 0;
                TORCH = 0;
                CHRG  = 1;

                if(!FLAG)
                    next_state = STATE_READY;
                    
                if(Button_Pressed(&GPIOD_IDR, 4, &buttonstate_up))
                    next_state = STATE_TORCH;

                break;

            case STATE_READY:
                Write_Mode_Text(0, 0, 0, 0, 1, CL_BLACK);
                if(Button_Pressed(&GPIOC_IDR, 13, &buttonstate_center))
                    next_state = STATE_FLASH;

                if(Button_Pressed(&GPIOD_IDR, 4, &buttonstate_up))
                    next_state = STATE_TORCH;

                break;

            case STATE_FLASH:
                Write_Mode_Text(0, 0, 0, 0, 1, CL_GRAY);
                CHRG = 0;

                FLASH = 1;
                Delay_ms(300);
                FLASH = 0;

                next_state = STATE_CHARGE;
                break;

            case STATE_TORCH:
                Write_Mode_Text(1, CL_BLACK, 1, CL_GRAY, 1, CL_GRAY);
                TORCH = 1;
                FLASH = 0;
                CHRG  = 0;

                if(Button_Pressed(&GPIOD_IDR, 4, &buttonstate_up))
                    next_state = STATE_CHARGE;

                break;
        }
        
        state = next_state;
        Delay_ms(POLLING_PERIOD);
    }
}

void MCU_Init() 
{
    // TORCH pin output.
    GPIO_Digital_Output( &GPIOA_ODR, _GPIO_PINMASK_0 );
    // CHRG pin output.
    GPIO_Digital_Output( &GPIOA_ODR, _GPIO_PINMASK_4 );
    // FLASH pin output.
    GPIO_Digital_Output( &GPIOC_ODR, _GPIO_PINMASK_2 );
    
    // FLAG pin input.
    GPIO_Digital_Input(&GPIOD_IDR, _GPIO_PINMASK_10);
    // Joystic CENTER button.
    GPIO_Digital_Input(&GPIOC_IDR, _GPIO_PINMASK_13);
    // Joystic UP button.
    GPIO_Digital_Input(&GPIOD_IDR, _GPIO_PINMASK_4);

    FLASH = 0;
    TORCH = 0;
    CHRG  = 0;

    display_init();
    
    Write_Mode_Text(1, CL_GRAY, 1, CL_GRAY, 1, CL_GRAY);

    TFT_Set_Font( &Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL );
    TFT_Write_Text( "Note: Use joystick UP button to change mode.", 19, 150 );
    TFT_Write_Text( "Use joystick CENTER button to FLASH when ready.", 19, 167 );
    
    TFT_Set_Font( &Verdana12x13_Regular, CL_RED, FO_HORIZONTAL );
    TFT_Write_Text( "CAUTION: Do not look directly at LED's.", 19, 200 );
    
    Delay_ms(100);
}

uint8_t Button_Pressed(unsigned int *port, unsigned int pin, uint8_t* state) 
{
    uint8_t new_state = Button(port, pin, 1, 1);
    if(*state == 0xFF && new_state == 0x00 ) 
    {
        *state = new_state;
        return 0xFF;
    }

    *state = new_state;
    return 0x00;
}

void Write_Text(unsigned char* text, unsigned x, unsigned y, unsigned int color)
{
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, color, FO_HORIZONTAL);
    TFT_Write_Text(text, x, y);
}

void Write_Mode_Text(
    uint8_t write_torch,
    unsigned int torch_color, 
    uint8_t write_flash,
    unsigned int flash_color,
    uint8_t write_ready,
    unsigned int ready_color
)
{
    if(write_torch) Write_Text("TORCH", TORCH_TEXT_X, MODE_TEXT_Y, torch_color);
    if(write_flash) Write_Text("FLASH", FLASH_TEXT_X, MODE_TEXT_Y, flash_color);
    if(write_ready) Write_Text("READY", READY_TEXT_X, MODE_TEXT_Y, ready_color);
}
/*************** END OF FUNCTIONS *********************************************/